/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


/*****************************************************************************  
 * This sketch draws the Japanese flag and changes color of the dot
 *****************************************************************************/

float dotSize;  // size of the flag's dot in pixels
float dotHue;   // color (hue in HSB) of the flag's dot


/* -------------------------------------------------------------------
 * The setup function is run once at the beginning of the application.
 * Here you setup any initial values of variables, settings, etc.
 * ------------------------------------------------------------------- */
void setup() {
  
  // the size function MUST be called at the beginning of the setup function
  // to determine the size of the application window
  size(600, 400);
  // fullScreen();  // comment the line above and uncomment this line to run the application in full screen
  
  // The dot's size is set to half of the window's height
  dotSize = height/2;
  
  // Change color mode from RGB to HSB with Hue in range [0,360), Saturation in [0, 100), Brightness in [0, 100)
  colorMode(HSB, 360, 100, 100);  
  
  // Set hue to 0°, i.e. red
  dotHue = 0;
}


/* -------------------------------------------------------------------
 * The draw function is run repeatedly until the application quits.
 * Here you code what is drawn at each frame of the running application. 
 * ------------------------------------------------------------------- */
void draw() {
  
  // manipulate hue value by adding +1 each frame and starting over at 0 when 360 is reached using the modulo operator (% 360)
  // ### you can change the + 1 to larger values to increase the steps on the color circle ###
  dotHue = (dotHue + 1) % 360;
  
  // draw the flag at each frame
  drawFlag();
  
  // pause for 1 milliseconds. ### increase the value to slow down color change ### 
  delay(1);
}



/* -------------------------------------------------------------------
 * The drawFlag function actually draws the parts of the flag.  
 * ------------------------------------------------------------------- */
void drawFlag() {
  
  // do not draw strokes
  noStroke();

  // drag flag background: First set flag background color, then draw rectangle
  fill(0, 0, 100);             // <-- colors are now given by HSB
  rect(0, 0, width, height);
  
  // draw dot: First set dot color, then draw an ellipse with identical size along x- and y-axis
  fill(dotHue, 100, 100);      // <-- colors are now given by HSB
  ellipse(width/2, height/2, dotSize, dotSize);
}
