// --- Linear animation functions ---

float linear(float val, float start, float stop) {
  val = constrain(val, 0.0, 1.0);
  return start + val*(stop - start);
}

PVector linear(float val, PVector start, PVector stop) {
  PVector result = start.copy();
  val = constrain(val, 0.0, 1.0);
  
  result.x += val*(stop.x - start.x); 
  result.y += val*(stop.y - start.y); 
  result.z += val*(stop.z - start.z); 
  
  return result;
}


// --- nonlinear animation functions ---

float nonlinear(float val, float pow, float start, float stop) { 
  val = constrain(val, 0.0, 1.0);  
  float s = pow(val,pow);
  return start + s*(stop - start);
}

float nonlinearInverse(float val, float pow, float start, float stop) { 
  val = constrain(val, 0.0, 1.0);  
  float s = 1 - pow(1-val, pow);
  return start + s*(stop - start); 
}

float nonlinearInOut(float val, float pow, float start, float stop) { 
  val = constrain(val, 0.0, 1.0);  
  
  if (val <= 0.5) {
    return nonlinear(val*2.0, pow, start, 0.5*(start + stop));
  }
  else {
    return nonlinearInverse((val-0.5)*2.0, pow, 0.5*(start + stop), stop);
  }
}


PVector nonlinear(float val, float pow, PVector start, PVector stop) {
  PVector result = start.copy();
  val = constrain(val, 0.0, 1.0);
  float s = pow(val,pow);
  
  result.x += s*(stop.x - start.x); 
  result.y += s*(stop.y - start.y); 
  result.z += s*(stop.z - start.z); 
  
  return result;
}

PVector nonlinearInverse(float val, float pow, PVector start, PVector stop) {
  PVector result = start.copy();
  val = constrain(val, 0.0, 1.0);
  float s = 1 - pow(1-val, pow);
  
  result.x += s*(stop.x - start.x); 
  result.y += s*(stop.y - start.y); 
  result.z += s*(stop.z - start.z); 
  
  return result;
}

PVector nonlinearInOut(float val, float pow, PVector start, PVector stop) { 
  val = constrain(val, 0.0, 1.0);  
  
  if (val <= 0.5) {
    return nonlinear(val*2.0, pow, start, PVector.add(start, stop).mult(0.5));
  }
  else {
    return nonlinearInverse((val-0.5)*2.0, pow, PVector.add(start, stop).mult(0.5), stop);
  }
}


float sineInOut(float val, float start, float stop) {
  val = constrain(val, 0.0, 1.0)*PI - 0.5*PI;
  
  return (0.5 + 0.5*sin(val))*(stop - start) + start;
}

PVector sineInOut(float val, PVector start, PVector stop) {
  val = constrain(val, 0.0, 1.0)*PI - 0.5*PI;
  
  return PVector.sub(stop, start).mult(0.5 + 0.5*sin(val)).add(start);
}
